from typing import Any, TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Artist, Track, Album, Cover, PlaylistId, Video, Chart, Vinyl, Playlist


@model
class BriefInfo(YandexMusicObject):
    """Класс, представляющий информацию об артисте.

    Attributes:
        artist (:obj:`yandex_music.Artist` | :obj:`None`): Артист.
        albums (:obj:`list` из :obj:`yandex_music.Album`): Альбомы.
        playlists (:obj:`list` из :obj:`yandex_music.Playlist`): Плейлисты.
        also_albums (:obj:`list` из :obj:`yandex_music.Album`): Сборники.
        last_release_ids (:obj:`list` из :obj:`int`): Уникальные идентификаторы последних выпущенных альбомов.
        last_releases (:obj:`list` из :obj:`yandex_music.Album`, optional): Последние выпущенные альбомы.
        popular_tracks (:obj:`list` из :obj:`yandex_music.Track`): Популярные треки.
        similar_artists (:obj:`list` из :obj:`yandex_music.Artist`): Похожие артисты.
        all_covers (:obj:`list` из :obj:`yandex_music.Cover`): Все обложки.
        concerts (:obj:`str`): Концерты (тест-кейс с ними потерялся, мало у кого есть).
        videos (:obj:`list` из :obj:`yandex_music.Video`): Видео.
        vinyls (:obj:`list` из :obj:`yandex_music.Vinyl`): Пластинки.
        has_promotions (:obj:`bool`): Рекламируется ли TODO.
        playlist_ids (:obj:`list` из :obj:`yandex_music.PlaylistId`): Уникальные идентификаторы плейлистов.
        tracks_in_chart (:obj:`list` из :obj:`yandex_music.Chart`, optional): Треки в чарте.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    artist: Optional['Artist']
    albums: List['Album']
    playlists: List['Playlist']
    also_albums: List['Album']
    last_release_ids: List[int]
    last_releases: List['Album']
    popular_tracks: List['Track']
    similar_artists: List['Artist']
    all_covers: List['Cover']
    concerts: Any
    videos: List['Video']
    vinyls: List['Vinyl']
    has_promotions: bool
    playlist_ids: List['PlaylistId']
    tracks_in_chart: List['Chart'] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.artist,
            self.albums,
            self.playlists,
            self.also_albums,
            self.last_release_ids,
            self.popular_tracks,
            self.similar_artists,
            self.all_covers,
            self.concerts,
            self.videos,
            self.vinyls,
            self.has_promotions,
            self.playlist_ids,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['BriefInfo']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.BriefInfo`: Информация об артисте.
        """
        if not data:
            return None

        data = super(BriefInfo, cls).de_json(data, client)
        from yandex_music import Artist, Track, Album, Cover, PlaylistId, Video, Chart, Vinyl, Playlist

        data['playlists'] = Playlist.de_list(data.get('playlists'), client)
        data['artist'] = Artist.de_json(data.get('artist'), client)
        data['similar_artists'] = Artist.de_list(data.get('similar_artists'), client)
        data['popular_tracks'] = Track.de_list(data.get('popular_tracks'), client)
        data['albums'] = Album.de_list(data.get('albums'), client)
        data['also_albums'] = Album.de_list(data.get('also_albums'), client)
        data['last_releases'] = Album.de_list(data.get('last_releases'), client)
        data['all_covers'] = Cover.de_list(data.get('all_covers'), client)
        data['playlist_ids'] = PlaylistId.de_list(data.get('playlist_ids'), client)
        data['videos'] = Video.de_list(data.get('videos'), client)
        data['tracks_in_chart'] = Chart.de_list(data.get('tracks_in_chart'), client)
        data['vinyls'] = Vinyl.de_list(data.get('vinyls'), client)

        return cls(client=client, **data)



from typing import List, Optional, TYPE_CHECKING

from yandex_music import Album, Artist, Playlist, YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client

de_list = {
    'album': Album.de_json,
    'playlist': Playlist.de_json,
}


@model
class Like(YandexMusicObject):
    """Класс, представляющий объект с отметкой "мне нравится".

    None:
        В поле `type` содержится одно из трёх значений: `artist`, `playlist`, `album`. Обозначает поле, в котором
        содержится информация.

    Attributes:
        type (:obj:`str`): Тип объекта с отметкой.
        id (:obj:`str`, optional): Уникальный идентификатор отметки.
        timestamp (:obj:`str`, optional): Дата и время добавления отметки.
        album (:obj:`yandex_music.Album`, optional): Понравившейся альбом.
        artist (:obj:`yandex_music.Artist`, optional): Понравившейся артист.
        playlist (:obj:`yandex_music.Playlist`, optional): Понравившейся плейлист.
        short_description (:obj:`str`, optional): Короткое описание.
        description (:obj:`str`, optional): Описание.
        is_premiere (:obj:`bool`, optional): Премьера ли.
        is_banner (:obj:`bool`, optional): Является ли баннером.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    id: Optional[str] = None
    timestamp: Optional[str] = None
    album: Optional['Album'] = None
    artist: Optional['Artist'] = None
    playlist: Optional['Playlist'] = None
    short_description: Optional[str] = None
    description: Optional[str] = None
    is_premiere: Optional[bool] = None
    is_banner: Optional[bool] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.type, self.timestamp, self.album, self.artist, self.playlist)

    @classmethod
    def de_json(cls, data: dict, client: 'Client', type_: str = None) -> Optional['Like']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            type_ (:obj:`str`, optional): Тип объекта с отметкой "мне нравится".
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Like`: Объект с отметкой "мне нравится".
        """
        if not data:
            return None

        data = super(Like, cls).de_json(data, client)

        if type_ == 'artist':
            if 'artist' not in data:
                temp_data = data.copy()
                data.clear()
                data[type_] = Artist.de_json(temp_data, client)
            else:
                data[type_] = Artist.de_json(data.get('artist'), client)
        else:
            data[type_] = de_list[type_](data.get(type_), client)

        data['type'] = type_

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client', type_: str = None) -> List['Like']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            type_ (:obj:`str`, optional): Тип объекта с отметкой "мне нравится".
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Like`: Объекты с отметкой "мне нравится".
        """
        if not data:
            return []

        likes = list()
        for like in data:
            likes.append(cls.de_json(like, client, type_))

        return likes



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Shot


@model
class ShotEvent(YandexMusicObject):
    """Класс, представляющий событие-шот перед началом следующего трека.

    Attributes:
        event_id (:obj:`str`): Уникальный идентификатор события.
        shots (:obj:`list` из :obj:`yandex_music.Shot`): Шоты от Алисы.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    event_id: str
    shots: List['Shot']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.event_id, self.shots)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ShotEvent']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ShotEvent`: Событие-шот перед началом следующего трека.
        """
        if not data:
            return None

        data = super(ShotEvent, cls).de_json(data, client)
        from yandex_music import Shot

        data['shots'] = Shot.de_list(data.get('shots'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List, Union

from yandex_music import YandexMusicObject
from yandex_music.utils import model
from yandex_music.exceptions import InvalidBitrateError

if TYPE_CHECKING:
    from yandex_music import (
        Client,
        Normalization,
        Major,
        Album,
        Artist,
        Supplement,
        DownloadInfo,
        User,
        MetaData,
        PoetryLoverMatch,
    )


@model
class Track(YandexMusicObject):
    """Класс, представляющий трек.

    Note:
        Известные значения поля `content_warning`: `explicit`.

        Известные значения поля `type`: `music`.

        Поля `can_publish`, `state`, `desired_visibility`, `filename`, `user_info` доступны только у треков что были
        загружены пользователем.

        Обычно у подкастов поле `remember_position == True`, а у треков `remember_position == False`.

        Полное описание можно получить используя :func:`Track.get_supplement`.

    Attributes:
        id (:obj:`int` | :obj:`str`): Уникальный идентификатор.
        title (:obj:`str`, optional): Название.
        available (:obj:`bool`, optional): Доступен ли для прослушивания.
        artists (:obj:`list` из :obj:`yandex_music.Artist`, optional): Исполнители.
        albums (:obj:`list` из :obj:`yandex_music.Album`, optional): Альбомы.
        available_for_premium_users (:obj:`bool`, optional): Доступен ли для пользователей с подпиской.
        lyrics_available (:obj:`bool`, optional): Доступен ли текст песни.
        poetry_lover_matches (:obj:`list` из :obj:`yandex_music.PoetryLoverMatch`, optional): Список слов TODO.
        best (:obj:`bool`, optional): Лучшей ли трек TODO.
        real_id (:obj:`int` | :obj:`str`, optional): TODO.
        og_image (:obj:`str`, optional): Ссылка на превью Open Graph.
        type (:obj:`str`, optional): Тип.
        cover_uri (:obj:`str`, optional): Ссылка на изображение с обложкой.
        major (:obj:`yandex_music.Major`, optional): Мейджор-лейбл.
        duration_ms (:obj:`int`, optional): Длительность трека в миллисекундах.
        storage_dir (:obj:`str`, optional): В какой папке на сервере хранится файл TODO.
        file_size (:obj:`int`, optional): Размер файла. TODO добавить единицу измерения.
        substituted (:obj:`yandex_music.Track`, optional): Замещённый трек.
        matched_track (:obj:`yandex_music.Track`, optional): Соответствующий трек TODO.
        normalization (:obj:`list` из :obj:`yandex_music.Normalization`, optional): Значения для нормализации трека.
        error (:obj:`str`, optional): Сообщение об ошибке.
        can_publish (:obj:`bool`, optional): Может ли быть опубликован.
        state (:obj:`str`, optional): Состояние, например, playable.
        desired_visibility (:obj:`str`, optional): Видимость трека.
        filename (:obj:`str`, optional): Название файла.
        user_info (:obj:`yandex_music.User`, optional): Информация о пользователе, который загрузил трек.
        meta_data (:obj:`yandex_music.MetaData`, optional): Информация о метаданных трека.
        regions (:obj:`list` из :obj:`str`, optional): Регион TODO.
        available_as_rbt (:obj:`bool`, optional): TODO.
        content_warning (:obj:`str`, optional): Тип откровенного контента.
        explicit (:obj:`bool`, optional): Содержит ли откровенный контент.
        preview_duration_ms (:obj:`int`, optional): TODO.
        available_full_without_permission (:obj:`bool`, optional): Доступен ли без подписки.
        version (:obj:`str`, optional): Версия.
        remember_position (:obj:`bool`, optional): Если :obj:`True`, то запоминатся последняя позиция прослушивания,
            иначе позиция не запоминается.
        background_video_uri (:obj:`str`, optional): Ссылка на видеошот.
        short_description (:obj:`str`, optional): Краткое опсание эпизода подкаста.
        is_suitable_for_children (:obj:`bool`, optional): Подходит ли для детей TODO.
        client (:obj:`yandex_music.Client`): Клиент Yandex Music.
    """

    id: Union[str, int]
    title: Optional[str] = None
    available: Optional[bool] = None
    artists: List['Artist'] = None
    albums: List['Album'] = None
    available_for_premium_users: Optional[bool] = None
    lyrics_available: Optional[bool] = None
    poetry_lover_matches: List['PoetryLoverMatch'] = None
    best: Optional[bool] = None
    real_id: Optional[Union[str, int]] = None
    og_image: Optional[str] = None
    type: Optional[str] = None
    cover_uri: Optional[str] = None
    major: Optional['Major'] = None
    duration_ms: Optional[int] = None
    storage_dir: Optional[str] = None
    file_size: Optional[int] = None
    substituted: Optional['Track'] = None
    matched_track: Optional['Track'] = None
    normalization: Optional['Normalization'] = None
    error: Optional[str] = None
    can_publish: Optional[bool] = None
    state: Optional[str] = None
    desired_visibility: Optional[str] = None
    filename: Optional[str] = None
    user_info: Optional['User'] = None
    meta_data: Optional['MetaData'] = None
    regions: Optional[List[str]] = None
    available_as_rbt: Optional[bool] = None
    content_warning: Optional[str] = None
    explicit: Optional[bool] = None
    preview_duration_ms: Optional[int] = None
    available_full_without_permission: Optional[bool] = None
    version: Optional[str] = None
    remember_position: Optional[bool] = None
    background_video_uri: Optional[str] = None
    short_description: Optional[str] = None
    is_suitable_for_children: Optional[bool] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self.download_info = None
        self._id_attrs = (self.id,)

    def get_download_info(self, get_direct_links=False) -> List['DownloadInfo']:
        """Сокращение для::

        client.tracks_download_info(self.track_id, get_direct_links)
        """
        self.download_info = self.client.tracks_download_info(self.track_id, get_direct_links)

        return self.download_info

    async def get_download_info_async(self, get_direct_links=False) -> List['DownloadInfo']:
        """Сокращение для::

        await client.tracks_download_info(self.track_id, get_direct_links)
        """
        self.download_info = await self.client.tracks_download_info(self.track_id, get_direct_links)

        return self.download_info

    def get_supplement(self, *args, **kwargs) -> Optional['Supplement']:
        """Сокращение для::

        client.track_supplement(track.id, *args, **kwargs)
        """
        return self.client.track_supplement(self.id, *args, **kwargs)

    async def get_supplement_async(self, *args, **kwargs) -> Optional['Supplement']:
        """Сокращение для::

        await client.track_supplement(track.id, *args, **kwargs)
        """
        return await self.client.track_supplement(self.id, *args, **kwargs)

    def download_cover(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    async def download_cover_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    def download_og_image(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Предпочтительнее использовать `self.download_cover()`.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    async def download_og_image_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Предпочтительнее использовать `self.download_cover_async()`.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    def download(self, filename: str, codec: str = 'mp3', bitrate_in_kbps: int = 192) -> None:
        """Загрузка трека.

        Note:
            Известные значения `codec`: `mp3`, `aac`.

            Известные значения `bitrate_in_kbps`: `64`, `128`, `192`, `320`.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            codec (:obj:`str`, optional): Кодек из доступных в `self.download_info`.
            bitrate_in_kbps (:obj:`int`, optional): Битрейт из доступных в `self.download_info` для данного кодека.

        Raises:
            :class:`yandex_music.exceptions.InvalidBitrateError`: Если в `self.download_info` не найден подходящий трек.
        """
        if self.download_info is None:
            self.get_download_info()

        for info in self.download_info:
            if info.codec == codec and info.bitrate_in_kbps == bitrate_in_kbps:
                info.download(filename)
                break
        else:
            raise InvalidBitrateError('Unavailable bitrate')

    async def download_async(self, filename: str, codec: str = 'mp3', bitrate_in_kbps: int = 192) -> None:
        """Загрузка трека.

        Note:
            Известные значения `codec`: `mp3`, `aac`.

            Известные значения `bitrate_in_kbps`: `64`, `128`, `192`, `320`.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            codec (:obj:`str`, optional): Кодек из доступных в `self.download_info`.
            bitrate_in_kbps (:obj:`int`, optional): Битрейт из доступных в `self.download_info` для данного кодека.

        Raises:
            :class:`yandex_music.exceptions.InvalidBitrateError`: Если в `self.download_info` не найден подходящий трек.
        """
        if self.download_info is None:
            await self.get_download_info_async()

        for info in self.download_info:
            if info.codec == codec and info.bitrate_in_kbps == bitrate_in_kbps:
                await info.download_async(filename)
                break
        else:
            raise InvalidBitrateError('Unavailable bitrate')

    def like(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_tracks_add(track.id, user.id, *args, **kwargs)
        """
        return self.client.users_likes_tracks_add(self.track_id, self.client.me.account.uid, *args, **kwargs)

    async def like_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_tracks_add(track.id, user.id, *args, **kwargs)
        """
        return await self.client.users_likes_tracks_add(self.track_id, self.client.me.account.uid, *args, **kwargs)

    def dislike(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_tracks_remove(track.id, user.id *args, **kwargs)
        """
        return self.client.users_likes_tracks_remove(self.track_id, self.client.me.account.uid, *args, **kwargs)

    async def dislike_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_tracks_remove(track.id, user.id *args, **kwargs)
        """
        return await self.client.users_likes_tracks_remove(self.track_id, self.client.me.account.uid, *args, **kwargs)

    def artists_name(self) -> List[str]:
        """Получает имена всех исполнителей.

        Returns:
              :obj:`list` из :obj:`str`: Имена исполнителей.
        """

        return [i.name for i in self.artists]

    @property
    def track_id(self) -> str:
        """:obj:`str`: Уникальный идентификатор трека состоящий из его номера и номера альбома или просто из номера."""
        if self.albums:
            return f'{self.id}:{self.albums[0].id}'
        return f'{self.id}'

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Track']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Track`: Трек.
        """
        if not data:
            return None

        data = super(Track, cls).de_json(data, client)
        from yandex_music import Normalization, Major, Album, Artist, User, MetaData, PoetryLoverMatch

        data['albums'] = Album.de_list(data.get('albums'), client)
        data['artists'] = Artist.de_list(data.get('artists'), client)
        data['normalization'] = Normalization.de_json(data.get('normalization'), client)
        data['major'] = Major.de_json(data.get('major'), client)
        data['substituted'] = Track.de_json(data.get('substituted'), client)
        data['matched_track'] = Track.de_json(data.get('matched_track'), client)
        data['user_info'] = User.de_json(data.get('user_info'), client)
        data['meta_data'] = MetaData.de_json(data.get('meta_data'), client)
        data['poetry_lover_matches'] = PoetryLoverMatch.de_list(data.get('poetry_lover_matches'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Track']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Track`: Треки.
        """
        if not data:
            return []

        return [cls.de_json(track, client) for track in data]

    # camelCase псевдонимы

    #: Псевдоним для :attr:`get_download_info`
    getDownloadInfo = get_download_info
    #: Псевдоним для :attr:`get_download_info_async`
    getDownloadInfoAsync = get_download_info_async
    #: Псевдоним для :attr:`get_supplement`
    getSupplement = get_supplement
    #: Псевдоним для :attr:`get_supplement_async`
    getSupplementAsync = get_supplement_async
    #: Псевдоним для :attr:`download_cover`
    downloadCover = download_cover
    #: Псевдоним для :attr:`download_cover_async`
    downloadCoverAsync = download_cover_async
    #: Псевдоним для :attr:`download_og_image`
    downloadOgImage = download_og_image
    #: Псевдоним для :attr:`download_og_image_async`
    downloadOgImageAsync = download_og_image_async
    #: Псевдоним для :attr:`track_id`
    trackId = track_id
    #: Псевдоним для :attr:`like_async`
    likeAsync = like_async
    #: Псевдоним для :attr:`dislike_async`
    dislike_async = dislike_async
    #: Псевдоним для :attr:`download_async`
    downloadAsync = download_async



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, PlaylistId


@model
class TagResult(YandexMusicObject):
    """Класс, представляющий тег и его плейлисты.

    Attributes:
        tag (:obj:`yandex_music.Tag`): Тег.
        ids (:obj:`list` из :obj:`yandex_music.PlaylistId`): Уникальные идентификаторы плейлистов тега.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    tag: str
    ids: List['PlaylistId']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.tag, self.ids)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['TagResult']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.TagResult`: Тег и его плейлисты.
        """
        if not data:
            return None

        data = super(TagResult, cls).de_json(data, client)
        from yandex_music import Tag, PlaylistId

        data['tag'] = Tag.de_json(data.get('tag'), client)
        data['ids'] = PlaylistId.de_list(data.get('ids'), client)

        return cls(client=client, **data)

    # TODO (MarshalX) add fetch_playlists shortcut?



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Cover(YandexMusicObject):
    """Класс, представляющий обложку.

    Attributes:
        type (:obj:`str`, optional): Тип обложки.
        uri (:obj:`str`, optional): Ссылка на изображение.
        items_uri (:obj:`str`, optional): Список ссылок на изображения.
        dir (:obj:`str`, optional): Директория хранения изображения на сервере.
        version (:obj:`str`, optional): Версия.
        is_custom (:obj:`bool`, optional): Является ли обложка пользовательской.
        custom (:obj:`bool`, optional): Является ли обложка пользовательской.
        prefix (:obj:`str`, optional): Уникальный идентификатор.
        copyright_name (:obj:`str`, optional): Название владельца авторским правом.
        copyright_cline (:obj:`str`, optional): Владелец прав на музыку (автор текста и т.д.), а не её записи.
        error (:obj:`str`, optional): Сообщение об ошибке.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: Optional[str] = None
    uri: Optional[str] = None
    items_uri: Optional[str] = None
    dir: Optional[str] = None
    version: Optional[str] = None
    custom: Optional[bool] = None
    is_custom: Optional[bool] = None
    copyright_name: Optional[str] = None
    copyright_cline: Optional[str] = None
    prefix: Optional[str] = None
    error: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.prefix, self.version, self.uri, self.items_uri)

    def download(self, filename: str, index: int = 0, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            index (:obj:`int`, optional): Индекс элемента в списке ссылок на обложки если нет `self.uri`.
            size (:obj:`str`, optional): Размер изображения.
        """
        uri = self.uri or self.items_uri[index]

        self.client.request.download(f'https://{uri.replace("%%", size)}', filename)

    async def download_async(self, filename: str, index: int = 0, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            index (:obj:`int`, optional): Индекс элемента в списке ссылок на обложки если нет `self.uri`.
            size (:obj:`str`, optional): Размер изображения.
        """
        uri = self.uri or self.items_uri[index]

        await self.client.request.download(f'https://{uri.replace("%%", size)}', filename)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Cover']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Cover`: Обложка.
        """
        if not data:
            return None

        data = super(Cover, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Cover']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Cover`: Обложки.
        """
        if not data:
            return []

        covers = list()
        for cover in data:
            covers.append(cls.de_json(cover, client))

        return covers

    # camelCase псевдонимы

    #: Псевдоним для :attr:`download_async`
    downloadAsync = download_async



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class PlaylistId(YandexMusicObject):
    """Класс, представляющий уникальный идентификатор плейлиста.

    Attributes:
        uid (:obj:`int`): Уникальный идентификатор пользователя владеющим плейлистом.
        kind (:obj:`int`): Уникальный идентификатор плейлиста.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    uid: int
    kind: int
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.uid, self.kind)

    @property
    def playlist_id(self):
        return f'{self.uid}:{self.kind}'

    def fetch_playlist(self, *args, **kwargs):
        """Сокращение для::

        client.users_playlists(kind, uid, *args, **kwargs)
        """
        return self.client.users_playlists(self.kind, self.uid, *args, **kwargs)

    async def fetch_playlist_async(self, *args, **kwargs):
        """Сокращение для::

        await client.users_playlists(kind, uid, *args, **kwargs)
        """
        return await self.client.users_playlists(self.kind, self.uid, *args, **kwargs)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['PlaylistId']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.PlaylistId`: Уникальный идентификатор плейлиста.
        """
        if not data:
            return None

        data = super(PlaylistId, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['PlaylistId']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.PlaylistId`: Уникальные идентификаторы плейлистов.
        """
        if not data:
            return []

        playlist_ids = list()
        for playlist_id in data:
            playlist_ids.append(cls.de_json(playlist_id, client))

        return playlist_ids

    # camelCase псевдонимы

    #: Псевдоним для :attr:`playlist_id`
    playlistId = playlist_id
    #: Псевдоним для :attr:`fetch_playlist`
    fetchPlaylist = fetch_playlist
    #: Псевдоним для :attr:`fetch_playlist_async`
    fetchPlaylistAsync = fetch_playlist_async



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Cover


@model
class OpenGraphData(YandexMusicObject):
    """Класс, представляющий данные для Open Graph.

    Attributes:
        title (:obj:`str`): Заголовок.
        description (:obj:`str`): Описание.
        image (:obj:`yandex_music.Cover`): Изображение.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    title: str
    description: str
    image: 'Cover'
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.title, self.description, self.image)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['OpenGraphData']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.OpenGraphData`: Данные для Open Graph.
        """
        if not data:
            return None

        data = super(OpenGraphData, cls).de_json(data, client)
        from yandex_music import Cover

        data['image'] = Cover.de_json(data.get('image'), client)

        return cls(client=client, **data)



####################################################################
# THIS IS AUTO GENERATED COPY OF client.py. DON'T EDIT IN BY HANDS #
####################################################################

import re
import logging
import keyword

from typing import TYPE_CHECKING, Optional, Union

# Не используется ujson из-за отсутствия в нём object_hook'a
# Отправка вообще application/x-www-form-urlencoded, а не JSON'a
# https://github.com/psf/requests/blob/master/requests/models.py#L508
import json

import asyncio
import aiohttp
import aiofiles

from yandex_music.utils.response import Response
from yandex_music.exceptions import (
    UnauthorizedError,
    BadRequestError,
    NetworkError,
    YandexMusicError,
    TimedOutError,
    NotFoundError,
)

if TYPE_CHECKING:
    from yandex_music import Client


USER_AGENT = 'Yandex-Music-API'
HEADERS = {
    'X-Yandex-Music-Client': 'YandexMusicAndroid/23020251',
}

reserved_names = keyword.kwlist + ['client']

logging.getLogger('urllib3').setLevel(logging.WARNING)


class Request:
    """Вспомогательный класс для yandex_music, представляющий методы для выполнения POST и GET запросов, скачивания
    файлов.

    Args:
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        headers (:obj:`dict`, optional): Заголовки передаваемые с каждым запросом.
        proxy_url (:obj:`str`, optional): Прокси.
    """

    def __init__(self, client=None, headers=None, proxy_url=None):
        self.headers = headers or HEADERS.copy()

        self.client = self.set_and_return_client(client)

        # aiohttp
        self.proxy_url = proxy_url

        # requests
        self.proxies = {'http': proxy_url, 'https': proxy_url} if proxy_url else None

    def set_language(self, lang: str) -> None:
        """Добавляет заголовок языка для каждого запроса.

        Note:
            Возможные значения `lang`: en/uz/uk/us/ru/kk/hy.

        Args:
            lang (:obj:`str`): Язык.
        """
        self.headers.update({'Accept-Language': lang})

    def set_authorization(self, token: str) -> None:
        """Добавляет заголовок авторизации для каждого запроса.

        Note:
            Используется при передаче своего экземпляра Request'a клиенту.

        Args:
            token (:obj:`str`): OAuth токен.
        """
        self.headers.update({'Authorization': f'OAuth {token}'})

    def set_and_return_client(self, client) -> 'Client':
        """Принимает клиент и присваивает его текущему объекту. При наличии авторизации добавляет заголовок.

        Args:
            client (:obj:`yandex_music.Client`): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Client`: Клиент Yandex Music.
        """
        self.client = client

        if self.client and self.client.token:
            self.set_authorization(self.client.token)

        return self.client

    @staticmethod
    def _convert_camel_to_snake(text: str) -> str:
        """Конвертация CamelCase в SnakeCase.

        Args:
            text (:obj:`str`): Название переменной в CamelCase.

        Returns:
            :obj:`str`: Название переменной в SnakeCase.
        """
        s = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', text)
        return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s).lower()

    @staticmethod
    def _object_hook(obj: dict) -> dict:
        """Нормализация имён переменных пришедших с API.

        Note:
            В названии переменной заменяет "-" на "_", конвертирует в SnakeCase, если название является
            зарезервированным словом или "client" - добавляет "_" в конец. Если название переменной начинается с цифры -
            добавляет в начало "_".

        Args:
            obj (:obj:`dict`): Словарь, где ключ название переменной, а значение - содержимое.

        Returns:
            :obj:`dict`: Тот же словарь, что и на входе, но с нормализованными ключами.
        """
        cleaned_object = {}
        for key, value in obj.items():
            key = Request._convert_camel_to_snake(key.replace('-', '_'))
            key = key.lower()

            if key in reserved_names:
                key += '_'

            if len(key) and key[0].isdigit():
                key = '_' + key

            cleaned_object.update({key: value})

        return cleaned_object

    def _parse(self, json_data: bytes) -> Optional[Response]:
        """Разбор ответа от API.

        Note:
            Если данные отсутствуют в `result`, то переформировывает ответ используя данные из корня.

        Args:
            json_data (:obj:`bytes`): Ответ от API.

        Returns:
            :obj:`yandex_music.utils.response.Response`: Ответ API.

        Raises:
            :class:`yandex_music.exceptions.YandexMusicError`: Базовое исключение библиотеки.
        """
        try:
            decoded_s = json_data.decode('utf-8')
            data = json.loads(decoded_s, object_hook=Request._object_hook)

        except UnicodeDecodeError:
            logging.getLogger(__name__).debug('Logging raw invalid UTF-8 response:\n%r', json_data)
            raise YandexMusicError('Server response could not be decoded using UTF-8')
        except (AttributeError, ValueError):
            raise YandexMusicError('Invalid server response')

        if data.get('result') is None:
            data = {'result': data, 'error': data.get('error'), 'error_description': data.get('error_description')}

        return Response.de_json(data, self.client)

    async def _request_wrapper(self, *args, **kwargs):
        """Обёртка над запросом библиотеки `aiohttp`.

        Note:
            Добавляет необходимые заголовки для запроса, обрабатывает статус коды, следит за таймаутом, кидает
            необходимые исключения, возвращает ответ. Передаёт пользовательские аргументы в запрос.

        Args:
            *args: Произвольные аргументы для `aiohttp.request`.
            **kwargs: Произвольные ключевые аргументы для `aiohttp.request`.

        Returns:
            :obj:`bytes`: Тело ответа.

        Raises:
            :class:`yandex_music.exceptions.TimedOutError`: При превышении времени ожидания.
            :class:`yandex_music.exceptions.UnauthorizedError`: При невалидном токене, долгом ожидании прямой ссылки на файл.
            :class:`yandex_music.exceptions.BadRequestError`: При неправильном запросе.
            :class:`yandex_music.exceptions.NetworkError`: При проблемах с сетью.
        """
        if 'headers' not in kwargs:
            kwargs['headers'] = {}

        kwargs['headers']['User-Agent'] = USER_AGENT

        try:
            async with aiohttp.request(*args, **kwargs) as _resp:
                resp = _resp
                content = await resp.content.read()
        except asyncio.TimeoutError:
            raise TimedOutError()
        except aiohttp.ClientError as e:
            raise NetworkError(e)

        if 200 <= resp.status <= 299:
            return content

        try:
            parse = self._parse(content)
            message = parse.get_error()
        except YandexMusicError:
            message = 'Unknown HTTPError'

        if resp.status in (401, 403):
            raise UnauthorizedError(message)
        elif resp.status == 400:
            raise BadRequestError(message)
        elif resp.status == 404:
            raise NotFoundError(message)
        elif resp.status in (409, 413):
            raise NetworkError(message)

        elif resp.status == 502:
            raise NetworkError('Bad Gateway')
        else:
            raise NetworkError(f'{message} ({resp.status}): {content}')

    async def get(
        self, url: str, params: dict = None, timeout: Union[int, float] = 5, *args, **kwargs
    ) -> Union[dict, str]:
        """Отправка GET запроса.

        Args:
            url (:obj:`str`): Адрес для запроса.
            params (:obj:`str`): GET параметры для запроса.
            timeout (:obj:`int` | :obj:`float`): Используется как время ожидания ответа от сервера вместо указанного
                при создании пула.
            *args: Произвольные аргументы для `aiohttp.request`.
            **kwargs: Произвольные ключевые аргументы для `aiohttp.request`.

        Returns:
            :obj:`dict` | :obj:`str`: Обработанное тело ответа.

        Raises:
            :class:`yandex_music.exceptions.YandexMusicError`: Базовое исключение библиотеки.
        """
        result = await self._request_wrapper(
            'GET',
            url,
            params=params,
            headers=self.headers,
            proxy=self.proxy_url,
            timeout=aiohttp.ClientTimeout(total=timeout),
            *args,
            **kwargs,
        )

        return self._parse(result).get_result()

    async def post(self, url, data=None, timeout=5, *args, **kwargs) -> Union[dict, str]:
        """Отправка POST запроса.

        Args:
            url (:obj:`str`): Адрес для запроса.
            data (:obj:`str`): POST тело запроса.
            timeout (:obj:`int` | :obj:`float`): Используется как время ожидания ответа от сервера вместо указанного
                при создании пула.
            *args: Произвольные аргументы для `aiohttp.request`.
            **kwargs: Произвольные ключевые аргументы для `aiohttp.request`.

        Returns:
            :obj:`dict` | :obj:`str`: Обработанное тело ответа.

        Raises:
            :class:`yandex_music.exceptions.YandexMusicError`: Базовое исключение библиотеки.
        """
        result = await self._request_wrapper(
            'POST',
            url,
            headers=self.headers,
            proxy=self.proxy_url,
            data=data,
            timeout=aiohttp.ClientTimeout(total=timeout),
            *args,
            **kwargs,
        )

        return self._parse(result).get_result()

    async def retrieve(self, url, timeout=5, *args, **kwargs) -> bytes:
        """Отправка GET запроса и получение содержимого без обработки (парсинга).

        Args:
            url (:obj:`str`): Адрес для запроса.
            timeout (:obj:`int` | :obj:`float`): Используется как время ожидания ответа от сервера вместо указанного
                при создании пула.
            *args: Произвольные аргументы для `aiohttp.request`.
            **kwargs: Произвольные ключевые аргументы для `aiohttp.request`.

        Returns:
            :obj:`bytes`: Тело ответа.

        Raises:
            :class:`yandex_music.exceptions.YandexMusicError`: Базовое исключение библиотеки.
        """
        return await self._request_wrapper(
            'GET', url, proxy=self.proxy_url, timeout=aiohttp.ClientTimeout(total=timeout), *args, **kwargs
        )

    async def download(self, url, filename, timeout=5, *args, **kwargs) -> None:
        """Отправка запроса на получение содержимого и его запись в файл.

        Args:
            url (:obj:`str`): Адрес для запроса.
            filename (:obj:`str`): Путь и(или) название файла вместе с расширением.
            timeout (:obj:`int` | :obj:`float`): Используется как время ожидания ответа от сервера вместо указанного
                при создании пула.
            *args: Произвольные аргументы для `aiohttp.request`.
            **kwargs: Произвольные ключевые аргументы для `aiohttp.request`.

        Raises:
            :class:`yandex_music.exceptions.YandexMusicError`: Базовое исключение библиотеки.
        """
        result = await self.retrieve(url, timeout=timeout, *args, *kwargs)
        async with aiofiles.open(filename, 'wb') as f:
            await f.write(result)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Price


@model
class Product(YandexMusicObject):
    """Класс, представляющий продаваемый продукт.

    Attributes:
        product_id (:obj:`str`): Уникальный идентификатор.
        type (:obj:`str`): Тип продаваемого.
        common_period_duration (:obj:`str`): Длительность общего периода.
        duration (:obj:`int`): Длительность.
        trial_duration (:obj:`int`): Длительность испытательного срока.
        price (:obj:`yandex_music.Price`): Цена.
        feature (:obj:`str`): Предоставляемая возможность.
        debug (:obj:`bool`): Отладочный продукт.
        plus (:obj:`bool`): Даёт ли подписку "Плюс".
        cheapest (:obj:`bool`, optional): Самый дешёвый (лучшее предложение).
        title (:obj:`str`, optional): Заголовок продукта.
        family_sub (:obj:`bool`, optional): Семейная ли подписка.
        fb_image (:obj:`str`, optional): Картинка для превью на facebook.
        fb_name (:obj:`str`, optional): Заголовок превью на facebook.
        family (:obj:`bool`, optional): Доступно ли для семьи.
        features (:obj:`list` из :obj:`str`, optional): Список предоставляемых возможностей.
        description (:obj:`str`, optional): Описание.
        available (:obj:`bool`, optional): Доступна ли покупка.
        trial_available (:obj:`bool`, optional): Доступен ли пробный период.
        trial_period_duration (:obj:`str`, optional): Длительность пробного периода.
        intro_period_duration (:obj:`str`, optional): Длительность вступительного периода TODO.
        intro_price (:obj:`yandex_music.Price`, optional): Цена вступительного периода.
        start_period_duration (:obj:`str`, optional): Длительность первого срока (за меньшую цену).
        start_price (:obj:`yandex_music.Price`, optional): Цена за первый срок.
        licence_text_parts (:obj:`list` из :obj:`yandex_music.LicenceTextPart`, optional):
            Длительность пробного периода.
        vendor_trial_available (:obj:`bool`, optional): Доступен испытательный срок продавца TODO.
        button_text (:obj:`str`, optional): Текст кнопки.
        button_additional_text (:obj:`str`, optional): Дополнительный текст кнопки.
        payment_method_types (:obj:`list` из :obj:`str`, optional): Способы оплаты.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    product_id: str
    type: str
    common_period_duration: str
    duration: int
    trial_duration: int
    price: Optional['Price']
    feature: str
    debug: bool
    plus: bool
    cheapest: Optional[bool] = None
    title: Optional[str] = None
    family_sub: Optional[bool] = None
    fb_image: Optional[str] = None
    fb_name: Optional[str] = None
    family: Optional[bool] = None
    features: List[str] = None
    description: Optional[str] = None
    available: Optional[bool] = None
    trial_available: Optional[bool] = None
    trial_period_duration: Optional[str] = None
    intro_period_duration: Optional[str] = None
    intro_price: Optional['Price'] = None
    start_period_duration: Optional[str] = None
    start_price: Optional['Price'] = None
    licence_text_parts: List['Price'] = None
    vendor_trial_available: Optional[bool] = None
    button_text: Optional[str] = None
    button_additional_text: Optional[str] = None
    payment_method_types: List[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.product_id,
            self.type,
            self.common_period_duration,
            self.duration,
            self.trial_duration,
            self.product_id,
            self.feature,
            self.debug,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Product']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Product`: Продаваемый продукт.
        """
        if not data:
            return None

        data = super(Product, cls).de_json(data, client)
        from yandex_music import Price, LicenceTextPart

        data['price'] = Price.de_json(data.get('price'), client)
        data['intro_price'] = Price.de_json(data.get('intro_price'), client)
        data['start_price'] = Price.de_json(data.get('start_price'), client)
        data['licence_text_parts'] = LicenceTextPart.de_list(data.get('licence_text_parts'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Product']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Product`: Продаваемые продукты.
        """
        if not data:
            return []

        return [cls.de_json(product, client) for product in data]



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, AutoRenewable, RenewableRemainder, NonAutoRenewable, Operator


@model
class Subscription(YandexMusicObject):
    """Класс, представляющий информацию о подписках пользователя.

    Attributes:
        non_auto_renewable_remainder (:obj:`yandex_music.RenewableRemainder`): Напоминание о продлении.
        auto_renewable (:obj:`list` из :obj:`yandex_music.AutoRenewable`, optional): Автопродление.
        family_auto_renewable (:obj:`list` из :obj:`yandex_music.AutoRenewable`): Автопродление семейной подписки.
        operator (:obj:`list` из :obj:`yandex_music.Operator`, optional): Услуги сотового оператора.
        non_auto_renewable (:obj:`yandex_music.NonAutoRenewable`, optional): Отключённое автопродление.
        can_start_trial (:obj:`bool`, optional): Есть ли возможность начать пробный период.
        mcdonalds (:obj:`bool`, optional): mcdonalds TODO.
        end (:obj:`str`, optional): Дата окончания.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    non_auto_renewable_remainder: 'RenewableRemainder'
    auto_renewable: List['AutoRenewable']
    family_auto_renewable: List['AutoRenewable']
    operator: List['Operator'] = None
    non_auto_renewable: Optional['NonAutoRenewable'] = None
    can_start_trial: Optional[bool] = None
    mcdonalds: Optional[bool] = None
    end: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.non_auto_renewable_remainder, self.auto_renewable, self.family_auto_renewable)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Subscription']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Subscription`: Информация о подписках пользователя.
        """
        if not data:
            return None

        data = super(Subscription, cls).de_json(data, client)
        from yandex_music import AutoRenewable, RenewableRemainder, NonAutoRenewable, Operator

        data['auto_renewable'] = AutoRenewable.de_list(data.get('auto_renewable'), client)
        data['family_auto_renewable'] = AutoRenewable.de_list(data.get('family_auto_renewable'), client)
        data['non_auto_renewable_remainder'] = RenewableRemainder.de_json(
            data.get('non_auto_renewable_remainder'), client
        )
        data['non_auto_renewable'] = NonAutoRenewable.de_json(data.get('non_auto_renewable'), client)
        data['operator'] = Operator.de_list(data.get('operator'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject, Playlist, ChartInfoMenu
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class ChartInfo(YandexMusicObject):
    """Класс, представляющий чарт.

    Attributes:
        id (:obj:`str`): Уникальный идентификатор блока.
        type (:obj:`str`): Тип блока.
        type_for_from (:obj:`str`): Откуда получен блок (как к нему пришли).
        title (:obj:`str`): Заголовок.
        menu (:obj:`yandex_music.ChartInfoMenu`, optional): Меню TODO.
        chart (:obj:`yandex_music.Playlist`, optional): Плейлист.
        chart_description (:obj:`str`, optional): Описание.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: str
    type: str
    type_for_from: str
    title: str
    menu: Optional['ChartInfoMenu']
    chart: Optional['Playlist']
    chart_description: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ChartInfo']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ChartInfo`: Чарт.
        """
        if not data:
            return None

        data = super(ChartInfo, cls).de_json(data, client)
        data['chart'] = Playlist.de_json(data.get('chart'), client)
        data['menu'] = ChartInfoMenu.de_json(data.get('menu'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List, Dict

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Title, Icon, Images


@model
class Genre(YandexMusicObject):
    """Класс, представляющий жанр музыки.

    Attributes:
        id (:obj:`str`): Уникальный идентификатор жанра.
        weight (:obj:`int`): Вес TODO (возможно, чем выше показатель, тем больше нравится пользователю).
        composer_top (:obj:`bool`): TODO.
        title (:obj:`str`): Заголовок жанра.
        titles (:obj:`dict`): Словарь заголовков на разных языках, где ключ - язык.
        images (:obj:`yandex_music.Images`): Изображение жанра.
        show_in_menu (:obj:`bool`): Показывать в меню.
        show_in_regions (:obj:`list` из :obj:`int`, optional): Список регионов в которых отображается жанр в списках.
        full_title (:obj:`str`, optional): Полный заголовок.
        url_part (:obj:`str`, optional): Часть ссылки на жанр для открытия в браузере.
        color (:obj:`str`, optional): Цвет фона изображения.
        radio_icon (:obj:`yandex_music.Icon`, optional): Иконка радио жанра.
        sub_genres (:obj:`list` из :obj:`yandex_music.Genre`, optional): Поджанры текущего жанра музыки.
        hide_in_regions (:obj:`list`, optional): В каких регионах скрывать жанр.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: str
    weight: int
    composer_top: bool
    title: str
    titles: Dict[str, Optional['Title']]
    images: Optional['Images']
    show_in_menu: bool
    show_in_regions: Optional[list] = None
    full_title: Optional[str] = None
    url_part: Optional[str] = None
    color: Optional[str] = None
    radio_icon: Optional['Icon'] = None
    sub_genres: List['Genre'] = None
    hide_in_regions = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.weight, self.composer_top, self.title, self.images, self.show_in_menu)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Genre']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Genre`: Жанр музыки.
        """
        if not data:
            return None

        data = super(Genre, cls).de_json(data, client)
        from yandex_music import Title, Icon, Images

        data['titles'] = Title.de_dict(data.get('titles'), client)
        data['images'] = Images.de_json(data.get('images'), client)
        data['radio_icon'] = Icon.de_json(data.get('radio_icon'), client)
        data['sub_genres'] = Genre.de_list(data.get('sub_genres'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Genre']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Genre`: Жанры музыки.
        """
        if not data:
            return []

        return [cls.de_json(genre, client) for genre in data]



from enum import Enum
from typing import List, Union

ujson = False
try:
    import ujson as json

    ujson = True
except ImportError:
    import json


class Operation(Enum):
    """Класс перечисления типов операций для изменения плейлиста.

    Note:
        Существует две операции: вставка, удаление.
    """

    INSERT = 'insert'
    DELETE = 'delete'


class Difference:
    """Класс, представляющий обёртку над созданием данных для запроса изменения плейлиста.

    Note:
        Результатом является перечень (массив) операций, которые будут применены к плейлисту.

        Конечной разницей (набором операций) является JSON, который будет отправлен в теле запроса.

    Attributes:
        operations (:obj:`list` из :obj:`dict`): Перечень операция для изменения плейлиста.
    """

    def __init__(self):
        self.operations = []

    def to_json(self) -> str:
        """Сериализация всех операций над плейлистом.

        Returns:
            :obj:`str`: Сформированное тело для запроса.
        """
        return json.dumps(self.operations, ensure_ascii=not ujson)

    def add_delete(self, from_: int, to: int) -> 'Difference':
        """Добавление операции удаления.

        Note:
            Передаётся диапазон для удаления треков.

        Args:
            from_ (:obj:`int`): С какого индекса.
            to (:obj:`int`): По какой индекс.

        Returns:
            :obj:`yandex_music.utils.difference.Difference`: Набор операций над плейлистом.
        """
        operation = {'op': Operation.DELETE.value, 'from': from_, 'to': to}

        self.operations.append(operation)
        return self

    def add_insert(self, at: int, tracks: Union[dict, List[dict]]) -> 'Difference':
        """Добавление операции вставки.

        Note:
            В `tracks` передаётся словарь с двумя ключами: `id`, `album_id`. Это нужно для формирования операции.

        Args:
            at (:obj:`int`): Индекс для вставки.
            tracks (:obj:`dict` | :obj:`list: из :obj:`dict`): Словарь уникальными идентификаторами треков.

        Returns:
            :obj:`yandex_music.utils.difference.Difference`: Набор операций над плейлистом.
        """
        # TODO принимать TrackId, а так же строку и сплитить её по ":". При отсутствии album_id кидать исключение.
        if not isinstance(tracks, list):
            tracks = [tracks]

        operation = {'op': Operation.INSERT.value, 'at': at, 'tracks': []}

        for track in tracks:
            track = type('TrackId', (), track)  # TODO replace to normal TrackId object

            operation['tracks'].append({'id': track.id, 'albumId': track.album_id})

        self.operations.append(operation)
        return self



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Track


@model
class Sequence(YandexMusicObject):
    """Класс, представляющий звено последовательности.

    Note:
        Известные значения поля `type_`: `track`. Возможно есть `ad`.

    Attributes:
        type (:obj:`str`): Тип звена.
        track (:obj:`yandex_music.Track` | :obj:`None`): Трек.
        liked (:obj:`bool`): Связанное ли.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    track: Optional['Track']
    liked: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.track, self.liked)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Sequence']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Sequence`: Звено последовательности.
        """
        if not data:
            return None

        data = super(Sequence, cls).de_json(data, client)
        from yandex_music import Track

        data['track'] = Track.de_json(data.get('track'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Sequence']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Sequence`: Последовательность треков.
        """
        if not data:
            return []

        sequences = list()
        for sequence in data:
            sequences.append(cls.de_json(sequence, client))

        return sequences



from typing import TYPE_CHECKING, List, Optional

from yandex_music import YandexMusicObject, ChartInfoMenuItem
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class ChartInfoMenu(YandexMusicObject):
    """Класс, представляющий меню чарта.

    Attributes:
        items (:obj:`list` из :obj:`yandex_music.ChartInfoMenuItem`): Список элементов меню.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    items: List['ChartInfoMenuItem']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.items,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ChartInfoMenu']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ChartInfoMenu`: Меню чарта.
        """
        if not data:
            return None

        data = super(ChartInfoMenu, cls).de_json(data, client)
        data['items'] = ChartInfoMenuItem.de_list(data.get('items'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Track


@model
class PlaylistRecommendations(YandexMusicObject):
    """Класс, представляющий рекомендации для плейлиста.

    Attributes:
        tracks (:obj:`list` из :obj:`yandex_music.Track`): Список рекомендованных треков.
        batch_id (:obj:`str`, optional): Уникальный идентификатор партии треков.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    tracks: List['Track']
    batch_id: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.batch_id, self.tracks)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['PlaylistRecommendations']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.PlaylistRecommendations`: Рекомендации для плейлиста.
        """
        if not data:
            return None

        data = super(PlaylistRecommendations, cls).de_json(data, client)
        from yandex_music import Track

        data['tracks'] = Track.de_list(data.get('tracks'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Deprecation(YandexMusicObject):
    """Класс, представляющий TODO.

    Attributes:
        target_album_id (:obj:`int`): Идентификатор альбома TODO.
        status (:obj:`str`): Состояние TODO.
        done (:obj:`str`): Выполнен ли TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    target_album_id: Optional[int]
    status: Optional[str]
    done: Optional[bool]
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.target_album_id, self.status, self.done)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Deprecation']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        Returns:
            :obj:`yandex_music.Deprecation`: TODO.
        """
        if not data:
            return None

        data = super(Deprecation, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Id(YandexMusicObject):
    """Класс, представляющий уникальный идентификатор станции.

    Note:
        Известные типы станций: `user`, `genre`.

    Attributes:
        type (:obj:`str`): Тип станции.
        tag (:obj:`str`): Тег станции.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        **kwargs: Произвольные ключевые аргументы полученные от API.
    """

    type: str
    tag: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.tag)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Id']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Id`: Уникальный идентификатор станции.
        """
        if not data:
            return None

        data = super(Id, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class LandingList(YandexMusicObject):
    """Класс, представляющий список объектов лендинга.

    Note:
        Известные значения поля `type`: `new-playlists`, `new-releases`, `podcasts`.

        В зависимости от типа будет заполнено то, или иное поле.

    Attributes:
        type (:obj:`str`): Тип результата.
        type_for_from (:obj:`str`): Откуда пришло событие.
        title (:obj:`str`): Заголовок страницы.
        id (:obj:`str`, optional): Уникальный идентификатор списка.
        new_releases (:obj:`list` из :obj:`int`, optional): Новые альбомы.
        new_playlists (:obj:`list` из :obj:`int`, optional): Новые плейлисты.
        podcasts (:obj:`list` из :obj:`int`, optional): Подкасты.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    type_for_from: str
    title: str
    id: Optional[str] = None
    new_releases: List[int] = None
    new_playlists: List[int] = None
    podcasts: List[int] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.new_releases, self.new_playlists, self.podcasts)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Chart']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.LandingList`: Список объектов лендинга.
        """
        if not data:
            return None

        data = super(LandingList, cls).de_json(data, client)
        from yandex_music import PlaylistId

        data['new_playlists'] = PlaylistId.de_list(data.get('new_playlists'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class User(YandexMusicObject):
    """Класс, представляющий пользователя.

    Note:
        Когда данный класс используется в `MadeFor` и `Playlist, то доступны все поля кроме `display_name` и
        `full_name`.

        При наличии экземпляра класса в `user_info` у `Track` (у самозагруженных треков) доступны только `uid`,
        '`login`, 'display_name` и `full_name`.

        Поле `regions` есть только при возвращении пользователей в результатах поисках.

    Attributes:
        uid (:obj:`int`): Идентификатор пользователя.
        login (:obj:`str`): Логин пользователя.
        name (:obj:`str`, optional): Имя пользователя.
        display_name (:obj:`str`, optional): Отображаемое пользователя.
        full_name (:obj:`str`, optional): Полное имя пользователя.
        sex (:obj:`str`, optional): Пол пользователя.
        verified (:obj:`bool`, optional): Участвует ли пользователь в генерации плейлистов дня и т.д., и т.п.
        regions (:obj:`list` из :obj:`int`, optional): Список регионов TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    uid: int
    login: str
    name: Optional[str] = None
    display_name: Optional[str] = None
    full_name: Optional[str] = None
    sex: Optional[str] = None
    verified: Optional[bool] = None
    regions: List[int] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.uid, self.login)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['User']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.User`: Пользователь.
        """
        if not data:
            return None

        data = super(User, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['User']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.User`: Пользователи.
        """
        if not data:
            return []

        return [cls.de_json(user, client) for user in data]



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Lyrics, VideoSupplement


@model
class Supplement(YandexMusicObject):
    """Класс, представляющий дополнительную информацию о треке.

    Attributes:
        id (:obj:`int`): Уникальный идентификатор дополнительной информации.
        lyrics (:obj:`yandex_music.Lyrics`): Текст песни.
        videos (:obj:`yandex_music.VideoSupplement`): Видео.
        radio_is_available (:obj:`bool`, optional): Доступно ли радио.
        description (:obj:`str`, optional): Полное описание эпизода подкаста.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: int
    lyrics: Optional['Lyrics']
    videos: List['VideoSupplement']
    radio_is_available: bool = None
    description: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.lyrics, self.videos)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Supplement']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Supplement`: Дополнительная информация о треке.
        """
        if not data:
            return None

        data = super(Supplement, cls).de_json(data, client)
        from yandex_music import Lyrics, VideoSupplement

        data['lyrics'] = Lyrics.de_json(data.get('lyrics'), client)
        data['videos'] = VideoSupplement.de_list(data.get('videos'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class RotorSettings(YandexMusicObject):
    """Класс, представляющий настройки станции.

    Note:
        Поля `energy`, `mood` используются в старых настройках (`settings1`).

        Значения `mood_energy`: `fun`, `active`, `calm`, `sad`, `all`.

        Значения `diversity`: `favorite`, `popular`, `discover`, `default`.

        Значения `language`: `not-russian`, `russian`, `any`.

    Attributes:
        language (:obj:`str`): Язык.
        diversity (:obj:`str`): Разнообразие (треки).
        mood (:obj:`int`, optional): Настроение (старое).
        energy (:obj:`int`, optional): Энергичное.
        mood_energy (:obj:`str`, optional): Настроение.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    language: str
    diversity: str
    mood: Optional[int] = None
    energy: Optional[int] = None
    mood_energy: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.language, self.diversity)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['RotorSettings']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.RotorSettings`: Настройки станции.
        """
        if not data:
            return None

        data = super(RotorSettings, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, InvocationInfo


@model
class Response(YandexMusicObject):
    """Класс, представляющий ответ API.

    Note:
        У ответа сервера два варианта возврата данных. Через корень (без вложенности, на уровне `invocation_info`)
        используется от силы пару раз. И в поле `result`. Второй считается основным.

        В `data` лежит копия всего ответа.

    Attributes:
        data (:obj:`dict`): Ответ на запрос. Используется тогда, когда отсутствует `result`.
        invocation_info (:obj:`yandex_music.InvocationInfo`, optional): Информация о запросе.
        result (:obj:`dict`, optional): Ответ на запрос (секция с результатом).
        error (:obj:`str`, optional): Код ошибки.
        error_description (:obj:`str`, optional): Описание ошибки.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    data: dict
    invocation_info: Optional['InvocationInfo'] = None
    result: dict = None
    error: str = None
    error_description: str = None
    client: Optional['Client'] = None

    def get_error(self) -> str:
        """:obj:`str`: Код ошибки вместе с описанием"""
        return f'{self.error} {self.error_description if self.error_description else ""}'

    def get_result(self) -> dict:
        """:obj:`dict`: Результат выполнения запроса. Данный для распаковки."""
        return self.data if self.result is None else self.result

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Response']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.utils.response.Response`: Ответ API.
        """
        if not data:
            return None

        data = super(Response, cls).de_json(data, client)
        data['data'] = data.copy()
        from yandex_music import InvocationInfo

        data['invocation_info'] = InvocationInfo.de_json(data.get('invocation_info'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, TrackId


@model
class TrackShortOld(YandexMusicObject):
    """Класс, представляющий сокращённую версию трека.

    Note:
        Данная версия менее богата полями и найдена позже первой, поэтому была принята как за старую версию.

        Другая версия сокращённого трека: :class:`yandex_music.TrackShort`.

    Attributes:
        track_id (:obj:`yandex_music.TrackId`): Уникальный идентификатор трека.
        timestamp (:obj:`str`): Дата TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    track_id: Optional['TrackId']
    timestamp: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.track_id,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['TrackShortOld']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.TrackShortOld`: Сокращённая версия трека или :obj:`None`.
        """
        if not data:
            return None

        data = super(TrackShortOld, cls).de_json(data, client)
        from yandex_music import TrackId

        data['track_id'] = TrackId.de_json(data.get('track_id'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['TrackShortOld']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.TrackShortOld`: Сокращённые версии треков.
        """
        if not data:
            return []

        tracks = list()
        for track in data:
            tracks.append(cls.de_json(track, client))

        return tracks



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Track, AlbumEvent, ArtistEvent


@model
class Event(YandexMusicObject):
    """Класс, представляющий событие фида.

    Note:
        Известные значения поля `type`: `tracks`, `artists`, `albums`, `notification`.

        Поле `message` заполнено только когда `type` равен `notification`.

        Примером значения поля `type_for_from` может служить `recommended-similar-artists`.

        Наличие данных в `tracks`, `albums`, `artists` напрямую зависит от `type`.

    Attributes:
        id (:obj:`str`): Уникальный идентификатор события.
        type (:obj:`str`): Тип события.
        type_for_from (:obj:`str`, optional): Откуда пришло событие.
        title (:obj:`str`, optional): Заголовок.
        tracks (:obj:`list` из :obj:`yandex_music.Track`, optional): Список треков.
        artists (:obj:`list` из :obj:`yandex_music.ArtistEvent`, optional): Список артистов с похожими и популярными
            треками.
        albums (:obj:`list` из :obj:`yandex_music.AlbumEvent`, optional): Список альбомов с треками.
        message (:obj:`str`, optional): Сообщение уведомления.
        device (:obj:`str`, optional): Устройство, с которого пришло уведомление.
        tracks_count (:obj:`int`, optional): Количество треков (возможно, уже не используется).
        genre (:obj:`str`, optional): Жанр треков.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: str
    type: str
    type_for_from: Optional[str] = None
    title: Optional[str] = None
    tracks: List['Track'] = None
    artists: List['ArtistEvent'] = None
    albums: List['AlbumEvent'] = None
    message: Optional[str] = None
    device: Optional[str] = None
    tracks_count: Optional[int] = None
    genre: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.type)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Event']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Event`: Событие фида.
        """
        if not data:
            return None

        data = super(Event, cls).de_json(data, client)
        from yandex_music import Track, AlbumEvent, ArtistEvent

        data['tracks'] = Track.de_list(data.get('tracks'), client)
        data['albums'] = AlbumEvent.de_list(data.get('albums'), client)
        data['artists'] = ArtistEvent.de_list(data.get('artists'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Event']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Event`: События фида.
        """
        if not data:
            return []

        events = list()
        for event in data:
            events.append(cls.de_json(event, client))

        return events



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Station, RotorSettings, AdParams


@model
class StationResult(YandexMusicObject):
    """Класс, представляющий радиостанцию с настройками.

    Attributes:
        station (:obj:`yandex_music.Station` | :obj:`None`): Станция.
        settings (:obj:`yandex_music.RotorSettings` | :obj:`None`): Первый набор настроек.
        settings2 (:obj:`yandex_music.RotorSettings` | :obj:`None`): Второй набор настроек.
        ad_params (:obj:`yandex_music.AdParams` | :obj:`None`): Настройки рекламы.
        explanation (:obj:`str`, optional): TODO.
        prerolls (:obj:`list` из :obj:`str`, optional): Прероллы TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    station: Optional['Station']
    settings: Optional['RotorSettings']
    settings2: Optional['RotorSettings']
    ad_params: Optional['AdParams']
    explanation: Optional[str] = None
    prerolls: Optional[list] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.station, self.settings, self.settings2, self.ad_params)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['StationResult']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.StationResult`: Радиостанция с настройками.
        """
        if not data:
            return None

        data = super(StationResult, cls).de_json(data, client)
        from yandex_music import Station, RotorSettings, AdParams

        data['station'] = Station.de_json(data.get('station'), client)
        data['settings'] = RotorSettings.de_json(data.get('settings'), client)
        data['settings2'] = RotorSettings.de_json(data.get('settings2'), client)
        data['ad_params'] = AdParams.de_json(data.get('ad_params'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['StationResult']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.StationResult`: Радиостанции с настройками.
        """
        if not data:
            return []

        station_results = list()
        for station_result in data:
            station_results.append(cls.de_json(station_result, client))

        return station_results



from typing import TYPE_CHECKING, Optional, Union

from yandex_music import YandexMusicObject, Artist, Album, Track, Playlist, Video, User
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


de_json_result = {
    'track': Track.de_json,
    'artist': Artist.de_json,
    'album': Album.de_json,
    'playlist': Playlist.de_json,
    'video': Video.de_json,
    'user': User.de_json,
    'podcast': Album.de_json,
    'podcast_episode': Track.de_json,
}


@model
class Best(YandexMusicObject):
    """Класс, представляющий лучший результат поиска.

    Attributes:
        type (:obj:`str`): Тип лучшего результата.
        result (:obj:`yandex_music.Track` | :obj:`yandex_music.Artist` | :obj:`yandex_music.Album` \
            | :obj:`yandex_music.Playlist` | :obj:`yandex_music.Video`): Лучший результат.
        text (:obj:`str`, optional): TODO.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    type: str
    result: Optional[Union[Track, Artist, Album, Playlist, Video]]
    text: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.type, self.result)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Best']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Best`: Лучший результат.
        """
        if not data:
            return None

        data = super(Best, cls).de_json(data, client)
        data['result'] = de_json_result.get(data.get('type'))(data.get('result'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class ChartInfoMenuItem(YandexMusicObject):
    """Класс, представляющий элемент меню чарта.

    Attributes:
        title (:obj:`str`): Заголовок.
        url (:obj:`str`): Постфикс для запроса чарта.
        selected (:obj:`bool`, optional): Текущий ли элемент.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    title: str
    url: str
    selected: Optional[bool] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.url, self.selected)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ChartInfoMenuItem']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ChartInfoMenuItem`: Элемент меню.
        """
        if not data:
            return None

        data = super(ChartInfoMenuItem, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: list, client: 'Client') -> List['ChartInfoMenuItem']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.ChartInfoMenuItem`: Список элементов меню чарта.
        """
        if not data:
            return []

        return [cls.de_json(item, client) for item in data]



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, TrackId


@model
class Chart(YandexMusicObject):
    """Класс, представляющий элемент чарта.

    Note:
        Смещение - это количество позиций, на которые трек поднялся или опустился в чарте.

    Attributes:
        position (:obj:`int`): Позиция.
        progress (:obj:`str`): TODO.
        listeners (:obj:`int`): Количество слушателей.
        shift (:obj:`int`): Смещение.
        bg_color (:obj:`str`, optional): Цвет заднего фона.
        track_id (:obj:`yandex_music.TrackId`, optional): Уникальный идентификатор трека.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    position: int
    progress: str
    listeners: int
    shift: int
    bg_color: Optional[str] = None
    track_id: Optional['TrackId'] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.position, self.progress, self.listeners, self.shift, self.track_id)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Chart']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Chart`: Элемент чарта.
        """
        if not data:
            return None

        data = super(Chart, cls).de_json(data, client)
        from yandex_music import TrackId

        data['track_id'] = TrackId.de_json(data.get('track_id'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Chart']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Chart`: Чарт.
        """
        if not data:
            return []

        return [cls.de_json(chart, client) for chart in data]



from typing import TYPE_CHECKING, Optional, List, Union

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, BlockEntity, PersonalPlaylistsData, PlayContextsData


@model
class Block(YandexMusicObject):
    """Класс, представляющий блок лендинга.

    Note:
        Известные значения поля `type_`: `personal-playlists`, `play-contexts`.

    Attributes:
        id (:obj:`str`): Уникальный идентификатор блока.
        type (:obj:`str`): Тип блока.
        type_for_from (:obj:`str`): Откуда получен блок (как к нему пришли).
        title (:obj:`str`): Заголовок.
        entities (:obj:`list` из :obj:`yandex_music.BlockEntity`): Содержимое блока (сущности, объекты).
        description (:obj:`str`, optional): Описание.
        data (:obj:`yandex_music.PersonalPlaylistsData` | :obj:`yandex_music.PlayContextsData`, optional):
            Дополнительные данные.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: str
    type: str
    type_for_from: str
    title: str
    entities: List['BlockEntity']
    description: Optional[str] = None
    data: Optional[Union['PersonalPlaylistsData', 'PlayContextsData']] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.type, self.type_for_from, self.title, self.entities)

    def __getitem__(self, item: int) -> 'BlockEntity':
        return self.entities[item]

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Block']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Block`: Блок лендинга.
        """
        if not data:
            return None

        data = super(Block, cls).de_json(data, client)
        from yandex_music import BlockEntity, PlayContextsData, PersonalPlaylistsData

        data['entities'] = BlockEntity.de_list(data.get('entities'), client)

        block_type = data.get('type')
        if block_type == 'personal-playlists':
            data['data'] = PersonalPlaylistsData.de_json(data.get('data'), client)
        elif block_type == 'play-contexts':
            data['data'] = PlayContextsData.de_json(data.get('data'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Block']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Block`: Блоки лендинга.
        """
        if not data:
            return []

        blocks = list()
        for block in data:
            blocks.append(cls.de_json(block, client))

        return blocks



from typing import TYPE_CHECKING, Optional, List, Union

from yandex_music import (
    YandexMusicObject,
    Promotion,
    Album,
    Playlist,
    MixLink,
    PlayContext,
    ChartItem,
    GeneratedPlaylist,
)
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client

de_json = {
    'personal-playlist': GeneratedPlaylist.de_json,
    'promotion': Promotion.de_json,
    'album': Album.de_json,
    'playlist': Playlist.de_json,
    'chart-item': ChartItem.de_json,
    'play-context': PlayContext.de_json,
    'mix-link': MixLink.de_json,
}


@model
class BlockEntity(YandexMusicObject):
    """Класс, представляющий содержимое блока.

    Note:
        В зависимости от поля `type_`, в поле `data` будет объект соответствующего типа.

        Известные значения поля `type_`: `personal-playlist`, `promotion`, `album`, `playlist`, `chart-item`,
        `play-context`, `mix-link`.

    Attributes:
        id (:obj:`str`): Уникальный идентификатор содержимого.
        type (:obj:`str`): Тип содержимого.
        data (:obj:`yandex_music.GeneratedPlaylist` | :obj:`yandex_music.Promotion` | :obj:`yandex_music.Album` |
            :obj:`yandex_music.Playlist` | :obj:`yandex_music.ChartItem` | :obj:`yandex_music.PlayContext`  |
            :obj:`yandex_music.MixLink`): Содержимое.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: str
    type: str
    data: Union['GeneratedPlaylist', 'Promotion', 'Album', 'Playlist', 'ChartItem', 'PlayContext', 'MixLink']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.type, self.data)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['BlockEntity']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.BlockEntity`: Сущность (объект) блока.
        """
        if not data:
            return None

        data = super(BlockEntity, cls).de_json(data, client)
        data['data'] = de_json.get(data.get('type'))(data.get('data'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['BlockEntity']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.BlockEntity`: Содержимое блока.
        """
        if not data:
            return []

        entities = list()
        for entity in data:
            entities.append(cls.de_json(entity, client))

        return entities



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Tag(YandexMusicObject):
    """Класс, представляющий тег (подборку).

    Attributes:
        id (:obj:`str`): Уникальный идентификатор тега.
        value (:obj:`str`): Значение тега (название в lower case).
        name (:obj:`str`): Название тега (отображаемое).
        og_description (:obj:`str`): Описание тега для OpenGraph.
        og_image (:obj:`str`, optional): Ссылка на изображение для OpenGraph.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: str
    value: str
    name: str
    og_description: str
    og_image: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Tag']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Tag`: Тег.
        """
        if not data:
            return None

        data = super(Tag, cls).de_json(data, client)

        return cls(client=client, **data)

    # TODO (MarshalX) add download_og_image shortcut?



from typing import TYPE_CHECKING, Optional, List, Union

from yandex_music.utils import model
from yandex_music import YandexMusicObject

if TYPE_CHECKING:
    from yandex_music import Client, Track, Chart


@model
class TrackShort(YandexMusicObject):
    """Класс, представляющий укороченную версию трека с неполными данными.

    Note:
        Поля `chart` и `track` только у треков, полученных через метод `chart()`.

    Attributes:
        id (:obj:`str`): Уникальный идентификатор трека.
        timestamp (:obj:`str`): Дата TODO.
        album_id (:obj:`str`, optional): Уникальный идентификатор альбома.
        play_count (:obj:`int`, optional): Количество проигрываний.
        recent (:obj:`bool`, optional): Недавний.
        chart (:obj:`yandex_music.Chart`, optional): Позиция в чарте.
        track (:obj:`yandex_music.Track`, optional): Полная версия трека.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: Union[str, int]
    timestamp: str
    album_id: Optional[str] = None
    play_count: Optional[int] = None
    recent: Optional[bool] = None
    chart: Optional['Chart'] = None
    track: Optional['Track'] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.album_id)

    def fetch_track(self) -> 'Track':
        """Получение полной версии трека.

        Returns:
            :obj:`yandex_music.Track`: Полная версия трека.
        """
        return self.client.tracks(self.track_id)[0]

    async def fetch_track_async(self) -> 'Track':
        """Получение полной версии трека.

        Returns:
            :obj:`yandex_music.Track`: Полная версия трека.
        """
        return (await self.client.tracks(self.track_id))[0]

    @property
    def track_id(self) -> str:
        """:obj:`str`: Уникальный идентификатор трека состоящий из его номера и номера альбома или просто из номера."""
        if self.album_id:
            return f'{self.id}:{self.album_id}'

        return f'{self.id}'

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['TrackShort']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.TrackShort`: Укороченная версия трека с неполными данными.
        """
        if not data:
            return None

        data = super(TrackShort, cls).de_json(data, client)
        from yandex_music import Track, Chart

        data['track'] = Track.de_json(data.get('track'), client)
        data['chart'] = Chart.de_json(data.get('chart'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['TrackShort']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.TrackShort`: Укороченные версии треков с неполными данными.
        """
        if not data:
            return []

        return [cls.de_json(track, client) for track in data]

    # camelCase псевдонимы

    #: Псевдоним для :attr:`fetch_track`
    fetchTrack = fetch_track
    #: Псевдоним для :attr:`fetch_track_async`
    fetchTrackAsync = fetch_track_async
    #: Псевдоним для :attr:`track_id`
    trackId = track_id



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, AlertButton


@model
class Alert(YandexMusicObject):
    """Класс, представляющий блок с предупреждением.

    Note:
        Данные предупреждения, скорее всего, только обкатываются. У них нет ID, вместо этого `xxx`, а еще присутствуют
        слова "тест" в тексте.

        Используют как предупреждение о конце подписки, так и о раздаче подарков.

    Attributes:
        alert_id (:obj:`str`): Уникальный идентификатор.
        text (:obj:`str`): Текст предупреждения.
        bg_color (:obj:`str`): Цвет заднего фона.
        text_color (:obj:`str`): Цвет текста.
        alert_type (:obj:`str`): Тип предупреждения.
        button (:obj:`yandex_music.AlertButton`): Кнопка с ссылкой.
        close_button (:obj:`bool`): Наличие кнопки "Закрыть".
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    alert_id: str
    text: str
    bg_color: str
    text_color: str
    alert_type: str
    button: 'AlertButton'
    close_button: bool
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.alert_id,)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Alert']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Alert`: Сообщение о статусе подписки.
        """
        if not data:
            return None

        from yandex_music import AlertButton

        data = super(Alert, cls).de_json(data, client)
        data['button'] = AlertButton.de_json(data.get('button'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class InvocationInfo(YandexMusicObject):
    """Класс, представляющий информацию о запросе.

    Attributes:
        hostname (:obj:`str`): Имя удалённого сервера.
        req_id (:obj:`str`): Номер запроса.
        exec_duration_millis (:obj:`str`, optional): Время выполнения в миллисекундах.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    hostname: str
    req_id: str
    exec_duration_millis: Optional[int] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.hostname, self.req_id)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['InvocationInfo']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.InvocationInfo`: Информация о запросе.
        """
        if not data:
            return None

        data = super(InvocationInfo, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class UserSettings(YandexMusicObject):
    """Класс, представляющий настройки пользователя.

    Note:
        Доступные значения для поля `theme`: `white`, `black`.

        Доступные значения для полей `user_music_visibility` и `user_social_visibility`: `private`, `public`.

    Notes:
        `promos_disabled`, `ads_disabled`, `rbt_disabled` устарели и не работают.

        `last_fm_scrobbling_enabled`, `facebook_scrobbling_enabled` выглядят устаревшими.

    Attributes:
        uid (:obj:`int`): Уникальный идентификатор пользователя.
        last_fm_scrobbling_enabled (:obj:`bool`): Скробблинг lastfm.
        shuffle_enabled (:obj:`bool`): Переключать треки в случайном порядке.
        volume_percents (:obj:`int`): Громкость звука в процентах.
        modified (:obj:`str`): Дата изменения настроек.
        facebook_scrobbling_enabled (:obj:`bool`): Скробблинг facebook.
        add_new_track_on_playlist_top (:obj:`bool`): Добавлять новые треки в начало плейлиста.
        user_music_visibility (:obj:`str`): Публичный доступ к моей фонотеке.
        user_social_visibility (:obj:`str`): Показывать соцсети на странице.
        rbt_disabled (:obj:`bool`): TODO (неиспользуемая фича).
        theme (:obj:`str`): Тема оформления.
        promos_disabled (:obj:`bool`): Не показывать рекламируемый контент).
        auto_play_radio (:obj:`bool`): Бесконечный поток музыки.
        sync_queue_enabled (:obj:`bool`): Синхронизация очередей между устройствами.
        ads_disabled (:obj:`bool`, optional): Не показывать рекламу.
        disk_enabled (:obj:`bool`, optional): TODO.
        show_disk_tracks_in_library (:obj:`bool`, optional): Показывать локальные треки в библиотеке.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    uid: int
    last_fm_scrobbling_enabled: bool
    shuffle_enabled: bool
    volume_percents: int
    modified: str
    facebook_scrobbling_enabled: bool
    add_new_track_on_playlist_top: bool
    user_music_visibility: str
    user_social_visibility: str
    rbt_disabled: bool
    theme: str
    promos_disabled: bool
    auto_play_radio: bool
    sync_queue_enabled: bool
    ads_disabled: Optional[bool] = None
    disk_enabled: Optional[bool] = None
    show_disk_tracks_in_library: Optional[bool] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.uid,
            self.last_fm_scrobbling_enabled,
            self.shuffle_enabled,
            self.volume_percents,
            self.modified,
            self.facebook_scrobbling_enabled,
            self.add_new_track_on_playlist_top,
            self.user_music_visibility,
            self.user_social_visibility,
            self.rbt_disabled,
            self.theme,
            self.promos_disabled,
            self.auto_play_radio,
            self.sync_queue_enabled,
            self.ads_disabled,
            self.disk_enabled,
            self.show_disk_tracks_in_library,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['UserSettings']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.UserSettings`: Настройки пользователя.
        """
        if not data:
            return None

        data = super(UserSettings, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List, Iterator

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Track, Pager


@model
class ArtistTracks(YandexMusicObject):
    """Класс, представляющий страницу списка треков артиста.

    Attributes:
        tracks (:obj:`list` из :obj:`yandex_music.Track`): Список треков артиста.
        pager (:obj:`yandex_music.Pager`): Пагинатор.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    tracks: List['Track']
    pager: Optional['Pager']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.pager, self.tracks)

    def __getitem__(self, item) -> 'Track':
        return self.tracks[item]

    def __iter__(self) -> Iterator['Track']:
        return iter(self.tracks)

    def __len__(self) -> int:
        return len(self.tracks)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ArtistTracks']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ArtistsTracks`: Страница списка треков артиста.
        """
        if not data:
            return None

        data = super(ArtistTracks, cls).de_json(data, client)
        from yandex_music import Track, Pager

        data['tracks'] = Track.de_list(data.get('tracks'), client)
        data['pager'] = Pager.de_json(data.get('pager'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, ShotType


@model
class ShotData(YandexMusicObject):
    """Класс, представляющий основную информацию о шоте.

    Attributes:
        cover_uri (:obj:`str`): Ссылка на обложку шота (иконка Алисы).
        mds_url (:obj:`str`): Ссылка на аудиоверсию шота в озвучке от Алисы.
        shot_text (:obj:`str`): Текстовая версия шота.
        shot_type (:obj:`yandex_music.ShotType`): Тип шота.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    cover_uri: str
    mds_url: str
    shot_text: str
    shot_type: 'ShotType'
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.cover_uri, self.mds_url, self.shot_text, self.shot_type)

    def download_cover(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    async def download_cover_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    def download_mds(self, filename: str) -> None:
        """Загрузка аудиоверсии шота.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
        """
        self.client.request.download(self.mds_url, filename)

    async def download_mds_async(self, filename: str) -> None:
        """Загрузка аудиоверсии шота.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
        """
        await self.client.request.download(self.mds_url, filename)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['ShotData']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.ShotData`: Основная информация о шоте.
        """
        if not data:
            return None

        data = super(ShotData, cls).de_json(data, client)
        from yandex_music import ShotType

        data['shot_type'] = ShotType.de_json(data.get('shot_type'), client)

        return cls(client=client, **data)

    # camelCase псевдонимы

    #: Псевдоним для :attr:`download_cover`
    downloadCover = download_cover
    #: Псевдоним для :attr:`download_cover_async`
    downloadCoverAsync = download_cover_async
    #: Псевдоним для :attr:`download_mds`
    downloadMds = download_mds
    #: Псевдоним для :attr:`download_mds_async`
    downloadMdsAsync = download_mds_async



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Value(YandexMusicObject):
    """Класс, представляющий значение(переменную).

    Attributes:
        value (:obj:`str`): Значение.
        name (:obj:`str`): Название.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    value: str
    name: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.value, self.name)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Value']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Value`: Значение.
        """
        if not data:
            return None

        data = super(Value, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Value']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Value`: Значения.
        """
        if not data:
            return []

        values = list()
        for value in data:
            values.append(cls.de_json(value, client))

        return values



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, GeneratedPlaylist, Day


@model
class Feed(YandexMusicObject):
    """Класс, представляющий фид.

    Note:
        Несмотря на то, что days это :obj:`list`, обычно возвращается только один день - текущий.

    Attributes:
        can_get_more_events (:obj:`bool`): Можно ли получить больше событий.
        pumpkin (:obj:`bool`): Хэллоуин.
        is_wizard_passed (:obj:`bool`): TODO.
        generated_playlists (:obj:`list` из :obj:`yandex_music.GeneratedPlaylist`): Сгенерированные плейлисты.
        headlines (:obj:`list` из :obj:`str`): Заголовки.
        today (:obj:`str`): Сегодняшняя дата в формате YYYY-MM-DD.
        days (:obj:`list` из :obj:`yandex_music.Day`): Дни.
        next_revision (:obj:`str`, optional): Дата следующих изменений в формате YYYY-MM-DD.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    can_get_more_events: bool
    pumpkin: bool
    is_wizard_passed: bool
    generated_playlists: List['GeneratedPlaylist']
    headlines: list
    today: str
    days: List['Day']
    next_revision: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.can_get_more_events, self.generated_playlists, self.headlines, self.today, self.days)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Feed']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Feed`: Фид.
        """
        if not data:
            return None

        data = super(Feed, cls).de_json(data, client)
        from yandex_music import GeneratedPlaylist, Day

        data['generated_playlists'] = GeneratedPlaylist.de_list(data.get('generated_playlists'), client)
        data['days'] = Day.de_list(data.get('days'), client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class TrackPosition(YandexMusicObject):
    """Класс, представляющий позицию трека.

    None:
        Позиция трека в альбоме, который возвращается при получении самого трека.

        Volume на фронте именуется как "Диск".

    Attributes:
        volume (:obj:`int`): Номер альбома.
        index (:obj:`int`): Порядковый номер трека.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    volume: int
    index: int
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.volume, self.index)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['TrackPosition']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.TrackPosition`: Позиция трека.
        """
        if not data:
            return None

        data = super(TrackPosition, cls).de_json(data, client)

        return cls(client=client, **data)



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Event, Track, TrackWithAds


@model
class Day(YandexMusicObject):
    """Класс, представляющий день в фиде.

    Attributes:
        day (:obj:`str`): Дата в формате YYYY-MM-DD.
        events (:obj:`list` из :obj:`yandex_music.Event`): События TODO.
        tracks_to_play_with_ads (:obj:`list` из :obj:`yandex_music.TrackWithAds`): Треки для проигрывания с рекламой.
        tracks_to_play (:obj:`list` из :obj:`yandex_music.Track`): Треки для проигрывания.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    day: str
    events: List['Event']
    tracks_to_play_with_ads: List['TrackWithAds']
    tracks_to_play: List['Track']
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.day, self.events, self.tracks_to_play_with_ads, self.tracks_to_play)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Day']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Day`: День в фиде.
        """
        if not data:
            return None

        data = super(Day, cls).de_json(data, client)
        from yandex_music import Event, Track, TrackWithAds

        data['events'] = Event.de_list(data.get('events'), client)
        data['tracks_to_play_with_ads'] = TrackWithAds.de_list(data.get('tracks_to_play_with_ads'), client)
        data['tracks_to_play'] = Track.de_list(data.get('tracks_to_play'), client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Day']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Day`: Дни в фиде.
        """
        if not data:
            return []

        days = list()
        for day in data:
            days.append(cls.de_json(day, client))

        return days



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class MixLink(YandexMusicObject):
    """Класс, представляющий ссылку (кликабельный блок) на подборку.

    Note:
        В цветах может как оказаться HEX (`#6c65a9`), так и какой-нибудь `transparent`.

        Ссылка со схемой отличается от просто ссылки наличием `yandexmusic://` в начале.

    Attributes:
        title (:obj:`str`): Заголовок ссылки.
        url (:obj:`str`): Ссылка на подборку.
        url_scheme (:obj:`str`): Ссылка со схемой на подборку.
        text_color (:obj:`str`): Цвет текста (HEX).
        background_color (:obj:`str`): Цвет заднего фона.
        background_image_uri (:obj:`str`): Ссылка на изображение заднего фона.
        cover_white (:obj:`str`): Ссылка на изображение с обложкой TODO.
        cover_uri (:obj:`str`, optional): Ссылка на изображение с обложкой.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    title: str
    url: str
    url_scheme: str
    text_color: str
    background_color: str
    background_image_uri: str
    cover_white: str
    cover_uri: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.url,
            self.title,
            self.url_scheme,
            self.text_color,
            self.background_color,
            self.background_image_uri,
            self.cover_white,
        )

    def download_background_image(self, filename: str, size: str = '200x200') -> None:
        """Загрузка заднего фона.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер заднего фона.
        """
        self.client.request.download(f'https://{self.background_image_uri.replace("%%", size)}', filename)

    async def download_background_image_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка заднего фона.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер заднего фона.
        """
        await self.client.request.download(f'https://{self.background_image_uri.replace("%%", size)}', filename)

    def download_cover_white(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки TODO.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.cover_white.replace("%%", size)}', filename)

    async def download_cover_white_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки TODO.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.cover_white.replace("%%", size)}', filename)

    def download_cover_uri(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    async def download_cover_uri_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['MixLink']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.MixLink`: Блок-ссылка на подборку.
        """
        if not data:
            return None

        data = super(MixLink, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['MixLink']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.MixLink`: Блоки-ссылки на подборки.
        """
        if not data:
            return []

        mix_links = list()
        for mix_link in data:
            mix_links.append(cls.de_json(mix_link, client))

        return mix_links

    # camelCase псевдонимы

    #: Псевдоним для :attr:`download_background_image`
    downloadBackgroundImage = download_background_image
    #: Псевдоним для :attr:`download_background_image_async`
    downloadBackgroundImageAsync = download_background_image_async
    #: Псевдоним для :attr:`download_cover_white`
    downloadCoverWhite = download_cover_white
    #: Псевдоним для :attr:`download_cover_white_async`
    downloadCoverWhiteAsync = download_cover_white_async
    #: Псевдоним для :attr:`download_cover_uri`
    downloadCoverUri = download_cover_uri
    #: Псевдоним для :attr:`download_cover_uri_async`
    downloadCoverUriAsync = download_cover_uri_async



from typing import TYPE_CHECKING, Optional, List, Union

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client


@model
class Label(YandexMusicObject):
    """Класс, представляющий лейбл альбома.

    Attributes:
        id (:obj:`int`): Идентификатор альбома.
        name (:obj:`str`): Название альбома.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: int
    name: str
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id, self.name)

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Label']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
        Returns:
            :obj:`yandex_music.Label`: Лейбл.
        """
        if not data:
            return None

        data = super(Label, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List[Union['Label', str]]:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Note:
            Лейблы строками возвращаются, как минимум, в результатах поиска. В остальных местах это объекты.

        Returns:
            :obj:`list` из :obj:`yandex_music.Label` или :obj:`str`: Лейблы.
        """
        if not data:
            return []

        labels = list()
        for label in data:
            if isinstance(label, dict):
                labels.append(cls.de_json(label, client))
            else:
                labels.append(label)

        return labels



from typing import TYPE_CHECKING, Optional

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Id, Icon, Restrictions


@model
class Station(YandexMusicObject):
    """Класс, представляющий станцию.

    Note:
        `id_for_from` обозначает предка станции, например, жанр, настроение или занятие.
        Неизвестно когда используется `id_for_from`, а когда `parent_id`.

    Attributes:
        id (:obj:`yandex_music.Id`): Уникальный идентификатор станции.
        name (:obj:`str`): Название станции.
        icon (:obj:`yandex_music.Icon`): Иконка станции.
        mts_icon (:obj:`yandex_music.Icon`): Иконка TODO.
        geocell_icon (:obj:`yandex_music.Icon`): Иконка TODO.
        id_for_from (:obj:`str`): Категория (тип) станции.
        restrictions (:obj:`yandex_music.Restrictions`): Ограничения для настроек станции старого формата.
        restrictions2 (:obj:`yandex_music.Restrictions`): Ограничения для настроек станции.
        full_image_url (:obj:`str`, optional): Ссылка на полное изображение.
        mts_full_image_url (:obj:`str`, optional): Ссылка на полную иконку.
        parent_id (:obj:`yandex_music.Id`, optional): Уникальный идентификатор станции, являющейся предком текущей.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: Optional['Id']
    name: str
    icon: 'Icon'
    mts_icon: 'Icon'
    geocell_icon: 'Icon'
    id_for_from: str
    restrictions: 'Restrictions'
    restrictions2: 'Restrictions'
    full_image_url: Optional[str] = None
    mts_full_image_url: Optional[str] = None
    parent_id: Optional['Id'] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (
            self.id,
            self.name,
            self.icon,
            self.mts_icon,
            self.geocell_icon,
            self.id_for_from,
            self.restrictions,
            self.restrictions2,
        )

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Station']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Station`: Станция.
        """
        if not data:
            return None

        data = super(Station, cls).de_json(data, client)
        from yandex_music import Id, Icon, Restrictions

        data['id'] = Id.de_json(data.get('id'), client)
        data['parent_id'] = Id.de_json(data.get('parent_id'), client)
        data['icon'] = Icon.de_json(data.get('icon'), client)
        data['mts_icon'] = Icon.de_json(data.get('mts_icon'), client)
        data['geocell_icon'] = Icon.de_json(data.get('geocell_icon'), client)
        data['restrictions'] = Restrictions.de_json(data.get('restrictions'), client)
        data['restrictions2'] = Restrictions.de_json(data.get('restrictions2'), client)

        return cls(client=client, **data)



from typing import Any, TYPE_CHECKING, Optional, List, Union

from yandex_music import YandexMusicObject
from yandex_music.utils import model

if TYPE_CHECKING:
    from yandex_music import Client, Artist, Label, TrackPosition, Track, Deprecation


@model
class Album(YandexMusicObject):
    """Класс, представляющий альбом.

    Note:
        Известные типы альбома: `single` - сингл, `compilation` - сборник.

        Известные предупреждения о содержимом: `explicit` - ненормативная лексика.

        Известные ошибки: `not-found` - альбом с таким ID не существует.

        Известные значения поля `meta_type`: `music`.

    Attributes:
        id_(:obj:`int`, optional): Идентификатор альбома.
        error (:obj:`str`, optional): Ошибка получения альбома.
        title (:obj:`str`, optional): Название альбома.
        track_count (:obj:`int`, optional): Количество треков.
        artists (:obj:`list` из :obj:`yandex_music.Artist`, optional): Артисты.
        labels (:obj:`list` из :obj:`yandex_music.Label` или :obj:`str`, optional): Лейблы.
        available (:obj:`bool`, optional): Доступен ли альбом.
        available_for_premium_users (:obj:`bool`, optional): Доступен ли альбом для пользователей с подпиской.
        version (:obj:`str`, optional): Дополнительная информация об альбоме.
        cover_uri (:obj:`str`, optional): Ссылка на обложку.
        content_warning (:obj:`str`, optional): Предупреждение о содержимом альбома.
        original_release_year: TODO.
        genre (:obj:`str`, optional): Жанр музыки.
        text_color (:obj:`str`, optional): Цвет текста описания.
        short_description (:obj:`str`, optional): Короткое описание.
        description (:obj:`str`, optional): Описание.
        is_premiere (:obj:`bool`, optional): Премьера ли.
        is_banner (:obj:`bool`, optional): Является ли баннером.
        meta_type (:obj:`str`, optional): Мета тип TODO.
        storage_dir (:obj:`str`, optional): В какой папке на сервере хранится файл TODO.
        og_image (:obj:`str`, optional): Ссылка на превью Open Graph.
        recent (:obj:`bool`, optional): Является ли альбом новым.
        very_important (:obj:`bool`, optional): Популярен ли альбом у слушателей.
        available_for_mobile (:obj:`bool`, optional): Доступен ли альбом из приложения для телефона.
        available_partially (:obj:`bool`, optional): Доступен ли альбом частично для пользователей без подписки.
        bests (:obj:`list` из :obj:`int`, optional): ID лучших треков альбома.
        duplicates (:obj:`list` из :obj:`yandex_music.Album`, optional): Альбомы-дубликаты.
        prerolls (:obj:`list`, optional): Прероллы TODO.
        volumes (:obj:`list` из :obj:`list` из :obj:`Track`, optional): Треки альбома, разделённые по дискам.
        year (:obj:`int`, optional): Год релиза.
        release_date (:obj:`str`, optional): Дата релиза в формате ISO 8601.
        type (:obj:`str`, optional): Тип альбома.
        track_position (:obj:`yandex_music.TrackPosition`, optional): Позиция трека в альбоме. Возвращается при
            получении альбома в составе трека.
        regions (:obj:`list` из :obj:`str`, optional): Список регионов в которых доступен альбом.
        available_as_rbt (:obj:`bool`, optional): TODO.
        lyrics_available (:obj:`bool`, optional): Доступны ли слова TODO.
        remember_position (:obj:`bool`, optional): Запоминание позиции TODO.
        albums (:obj:`list` из :obj:`yandex_music.Album`, optional): Альбомы TODO.
        duration_ms (:obj:`int`, optional): Длительность в миллисекундах.
        explicit (:obj:`bool`, optional): Есть ли в треке ненормативная лексика.
        start_date (:obj:`str`, optional): Дата начала в формате ISO 8601 TODO.
        likes_count (:obj:`int`, optional): Количество лайков TODO.
        deprecation (:obj:`yandex_music.Deprecation`, optional): TODO.
        available_regions (:obj:`list` из :obj:`str`, optional): Регионы, где доступн альбом.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: Optional[int] = None
    error: Optional[str] = None
    title: Optional[str] = None
    track_count: Optional[int] = None
    artists: List['Artist'] = None
    labels: List[Union['Label', str]] = None
    available: Optional[bool] = None
    available_for_premium_users: Optional[bool] = None
    version: Optional[str] = None
    cover_uri: Optional[str] = None
    content_warning: Optional[str] = None
    original_release_year: Any = None
    genre: Optional[str] = None
    text_color: Optional[str] = None
    short_description: Optional[str] = None
    description: Optional[str] = None
    is_premiere: Optional[bool] = None
    is_banner: Optional[bool] = None
    meta_type: Optional[str] = None
    storage_dir: Optional[str] = None
    og_image: Optional[str] = None
    buy: Optional[list] = None
    recent: Optional[bool] = None
    very_important: Optional[bool] = None
    available_for_mobile: Optional[bool] = None
    available_partially: Optional[bool] = None
    bests: Optional[List[int]] = None
    duplicates: List['Album'] = None
    prerolls: Optional[list] = None
    volumes: Optional[List[List['Track']]] = None
    year: Optional[int] = None
    release_date: Optional[str] = None
    type: Optional[str] = None
    track_position: Optional['TrackPosition'] = None
    regions: Optional[List[str]] = None
    available_as_rbt: Optional[bool] = None
    lyrics_available: Optional[bool] = None
    remember_position: Optional[bool] = None
    albums: Optional[List['Album']] = None
    duration_ms: Optional[int] = None
    explicit: Optional[bool] = None
    start_date: Optional[str] = None
    likes_count: Optional[int] = None
    deprecation: Optional['Deprecation'] = None
    available_regions: Optional[List[str]] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.id,)

    def with_tracks(self, *args, **kwargs) -> Optional['Album']:
        """Сокращение для::

        client.albums_with_tracks(album.id, *args, **kwargs)
        """
        return self.client.albums_with_tracks(self.id, *args, **kwargs)

    async def with_tracks_async(self, *args, **kwargs) -> Optional['Album']:
        """Сокращение для::

        await client.albums_with_tracks(album.id, *args, **kwargs)
        """
        return await self.client.albums_with_tracks(self.id, *args, **kwargs)

    def download_cover(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    async def download_cover_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.cover_uri.replace("%%", size)}', filename)

    def download_og_image(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Предпочтительнее использовать `self.download_cover()`.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    async def download_og_image_async(self, filename: str, size: str = '200x200') -> None:
        """Загрузка обложки.

        Предпочтительнее использовать `self.download_cover_async()`.

        Args:
            filename (:obj:`str`): Путь для сохранения файла с названием и расширением.
            size (:obj:`str`, optional): Размер обложки.
        """
        await self.client.request.download(f'https://{self.og_image.replace("%%", size)}', filename)

    def like(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_albums_add(album.id, user.id *args, **kwargs)
        """
        return self.client.users_likes_albums_add(self.id, self.client.me.account.uid, *args, **kwargs)

    async def like_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_albums_add(album.id, user.id *args, **kwargs)
        """
        return await self.client.users_likes_albums_add(self.id, self.client.me.account.uid, *args, **kwargs)

    def dislike(self, *args, **kwargs) -> bool:
        """Сокращение для::

        client.users_likes_albums_remove(album.id, user.id *args, **kwargs)
        """
        return self.client.users_likes_albums_remove(self.id, self.client.me.account.uid, *args, **kwargs)

    async def dislike_async(self, *args, **kwargs) -> bool:
        """Сокращение для::

        await client.users_likes_albums_remove(album.id, user.id *args, **kwargs)
        """
        return await self.client.users_likes_albums_remove(self.id, self.client.me.account.uid, *args, **kwargs)

    def artists_name(self) -> List[str]:
        """Получает имена всех исполнителей.

        Returns:
              :obj:`list` из :obj:`str`: Имена исполнителей.
        """

        return [i.name for i in self.artists]

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['Album']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.Album`: Альбом.
        """
        if not data:
            return None

        data = super(Album, cls).de_json(data, client)
        from yandex_music import Artist, Label, TrackPosition, Track, Deprecation

        data['artists'] = Artist.de_list(data.get('artists'), client)
        data['labels'] = Label.de_list(data.get('labels'), client)
        data['track_position'] = TrackPosition.de_json(data.get('track_position'), client)
        data['duplicates'] = Album.de_list(data.get('duplicates'), client)
        data['albums'] = Album.de_list(data.get('albums'), client)
        data['deprecation'] = Deprecation.de_json(data.get('deprecation'), client)
        if data.get('volumes'):
            data['volumes'] = [Track.de_list(i, client) for i in data['volumes']]

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['Album']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.Album`: Альбомы.
        """
        if not data:
            return []

        return [cls.de_json(album, client) for album in data]

    # camelCase псевдонимы

    #: Псевдоним для :attr:`with_tracks`
    withTracks = with_tracks
    #: Псевдоним для :attr:`with_tracks_async`
    withTracksAsync = with_tracks_async
    #: Псевдоним для :attr:`download_cover`
    downloadCover = download_cover
    #: Псевдоним для :attr:`download_cover_async`
    downloadCoverAsync = download_cover_async
    #: Псевдоним для :attr:`download_og_image`
    downloadOgImage = download_og_image
    #: Псевдоним для :attr:`download_og_image_async`
    downloadOgImageAsync = download_og_image_async
    #: Псевдоним для :attr:`artists_name`
    artistsName = artists_name



from typing import TYPE_CHECKING, Optional, List

from yandex_music import YandexMusicObject
from yandex_music.utils import model


if TYPE_CHECKING:
    from yandex_music import Client, Track


@model
class TrackId(YandexMusicObject):
    """Класс, представляющий уникальный идентификатор трека.

    Note:
        Поле `track_id` используется только у объектах полученных через очередь треков. В остальные случаях `id`.

        Поле `from_` есть только у объект, которые используются в очереди треков.

    Attributes:
        id (:obj:`int`): Уникальный идентификатор трека.
        track_id (:obj:`int`): Уникальный идентификатор трека.
        album_id (:obj:`int`, optional): Уникальный идентификатор альбома.
        from_ (:obj:`str`, optional): Откуда был получен этот объект.
        client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.
    """

    id: Optional[int] = None
    track_id: Optional[int] = None
    album_id: Optional[int] = None
    from_: Optional[str] = None
    client: Optional['Client'] = None

    def __post_init__(self):
        self._id_attrs = (self.track_id, self.id, self.album_id)

    @property
    def track_full_id(self) -> str:
        """:obj:`str`: ID трека состоящий из его номера и номера альбома."""
        track_id = self.id
        if self.track_id:
            track_id = self.track_id

        return f'{track_id}:{self.album_id}'

    def fetch_track(self, *args, **kwargs) -> 'Track':
        """Получение полной версии трека.

        Returns:
            :obj:`yandex_music.Track`: Полная версия.
        """
        return self.client.tracks(self.track_full_id, *args, **kwargs)[0]

    async def fetch_track_async(self, *args, **kwargs) -> 'Track':
        """Получение полной версии трека.

        Returns:
            :obj:`yandex_music.Track`: Полная версия.
        """
        return (await self.client.tracks(self.track_full_id, *args, **kwargs))[0]

    @classmethod
    def de_json(cls, data: dict, client: 'Client') -> Optional['TrackId']:
        """Десериализация объекта.

        Args:
            data (:obj:`dict`): Поля и значения десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`yandex_music.TrackId`: Уникальный идентификатор трека.
        """
        if not data:
            return None

        data = super(TrackId, cls).de_json(data, client)

        return cls(client=client, **data)

    @classmethod
    def de_list(cls, data: dict, client: 'Client') -> List['TrackId']:
        """Десериализация списка объектов.

        Args:
            data (:obj:`list`): Список словарей с полями и значениями десериализуемого объекта.
            client (:obj:`yandex_music.Client`, optional): Клиент Yandex Music.

        Returns:
            :obj:`list` из :obj:`yandex_music.TrackId`: Уникальные идентификаторы треков.
        """
        if not data:
            return []

        return [cls.de_json(track_id, client) for track_id in data]

    # camelCase псевдонимы

    #: Псевдоним для :attr:`fetch_track`
    fetchTrack = fetch_track
    #: Псевдоним для :attr:`fetch_track_async`
    fetchTrackAsync = fetch_track_async
    #: Псевдоним для :attr:`track_full_id`
    trackFullId = track_full_id
